@@ -42,6 +42,7 @@ List* db_top_get_all_project_infos(Database* db, List* project_infos)
 
 ERROR_LABEL(db->handle)
 }
+
 ProjectInfo* db_top_get_project_info(Database* db, ProjectInfo* project_info, char* project_name, bool all)
 {
     String* sql_a = string_new();
@@ -68,6 +69,7 @@ ProjectInfo* db_top_get_project_info(Database* db, ProjectInfo* project_info, ch
 
 ERROR_LABEL(db->handle)
 }
+
 void db_top_update_project(Database* db, Project* project)
 {
     exec_query(db, "update setting set value = ? where name = 'project_name'",
@@ -83,6 +85,7 @@ void db_top_update_project(Database* db, Project* project)
             COLUMN_TYPE_TEXT, string_rawstr(project->locale),
             COLUMN_TYPE_END);
 }
+
 void db_top_update_project_infos(Database* db, List* project_infos)
 {
     Iterator* it;
@@ -98,6 +101,7 @@ void db_top_update_project_infos(Database* db, List* project_infos)
     }
     return;
 }
+
 int db_top_register_project_info(Database* db, ProjectInfo* project_info)
 {
     exec_query(db, "insert into project_info(id, name, sort) values (NULL, ?, ?)",
@@ -107,6 +111,7 @@ int db_top_register_project_info(Database* db, ProjectInfo* project_info)
     
     return sqlite3_last_insert_rowid(db->handle);
 }
+
 char* db_top_get_project_db_name(char* project_name, char* buffer)
 {
     Database* top_db_a = db_init("top");
@@ -122,6 +127,7 @@ char* db_top_get_project_db_name(char* project_name, char* buffer)
     db_finish(top_db_a);
     return buffer;
 }
+
 String* get_search_sql_string_per_project(Database* db, DbInfo* db_info, List* keywords, String* sql_string)
 {
     string_appendf(sql_string,
@@ -150,6 +156,7 @@ String* get_search_sql_string_per_project(Database* db, DbInfo* db_info, List* k
     d("sql: %s\n", string_rawstr(sql_string));
     return sql_string;
 }
+
 void set_project_code_by_ticket(Ticket* t, List* projects)
 {
     Iterator* it;
@@ -161,32 +168,25 @@ void set_project_code_by_ticket(Ticket* t, List* projects)
         }
     }
 }
-List* db_top_search(Database* db, char* q, List* tickets)
+
+List* get_result_part(Database* db, List* projects, List* sqls, List* keywords, int start, int offset, List* tickets)
 {
-    List* projects_a;
+    int r, count = 0;
+    int index = 1;
+    sqlite3_stmt *stmt = NULL;
     Iterator* it;
     List* db_infos_a;
-    List* sqls_a;
     String* sql_search_a = string_new();
-    sqlite3_stmt *stmt = NULL;
-    List* keywords_a;
-    int i, r;
-    int index = 1;
 
-    list_alloc(keywords_a, String, string_new, string_free);
-    keywords_a = parse_keywords(keywords_a, q);
-    /* プロジェクトの検索 */
     list_alloc(db_infos_a, DbInfo, NULL, NULL);
-    list_alloc(projects_a, ProjectInfo, project_info_new, project_info_free);
-    list_alloc(sqls_a, String, string_new, string_free);
-    projects_a = db_top_get_all_project_infos(db, projects_a);
-    foreach (it, projects_a) {
+    foreach (it, projects) {
         char sql[DEFAULT_LENGTH];
         char db_name[DEFAULT_LENGTH];
         ProjectInfo* pi = it->element;
 
-        if (pi->id == 1) continue; /* トップなので飛ばす。 */
-        if (pi->deleted) continue; /* 削除されたサブプロジェクトなので飛ばす。 */
+        if (count < start || count > start + offset) continue; /* ignore cause out of target range. */
+        if (pi->id == 1) continue; /* ignore cause top sub project. */
+        if (pi->deleted) continue; /* ignore cause deleted sub project. */
         sprintf(sql, "attach ? as db%d", pi->id);
         d("%s\n", sql);
         sprintf(db_name, "db/%d.db", pi->id);
@@ -199,19 +199,20 @@ List* db_top_search(Database* db, char* q, List* tickets)
 
         if (SQLITE_ROW == sqlite3_step(stmt)) {
             DbInfo* db_info = list_new_element(db_infos_a);
-            String* s = list_new_element(sqls_a);
+            String* s = list_new_element(sqls);
             db_info->id = pi->id;
             db_info->field_count = sqlite3_column_int(stmt, 0);
             list_add(db_infos_a, db_info);
-            /* 検索用sql作成 */
-            s = get_search_sql_string_per_project(db, db_info, keywords_a, s);
-            list_add(sqls_a, s);
+            /* create search sql statement. */
+            s = get_search_sql_string_per_project(db, db_info, keywords, s);
+            list_add(sqls, s);
         }
         sqlite3_finalize(stmt);
+        count++;
     }
-    /* sqlを合成する。 */
+    /* merge sql statements. */
     string_append(sql_search_a, "select project_id, id from (\n");
-    foreach (it, sqls_a) {
+    foreach (it, sqls) {
         String* sql = it->element;
         string_append(sql_search_a, string_rawstr(sql));
         if (iterator_next(it))
@@ -221,48 +222,84 @@ List* db_top_search(Database* db, char* q, List* tickets)
     d("%s\n", string_rawstr(sql_search_a));
     if (sqlite3_prepare(db->handle, string_rawstr(sql_search_a), string_len(sql_search_a), &stmt, NULL) == SQLITE_ERROR) goto error;
     sqlite3_reset(stmt);
-    foreach (it, db_infos_a) { /* DBの数だけ */
+    /* setting parameters. */
+    foreach (it, db_infos_a) { /* for each dbs. */
         DbInfo* db_info = it->element;
         Iterator* it_k;
-        foreach (it_k, keywords_a) { /* キーワードの数だけ */
+        foreach (it_k, keywords) { /* for each keywords. */
+            int i;
             String* k = it_k->element;
-            for (i = 0; i < db_info->field_count; i++) { /* カラムの数だけ */
+            for (i = 0; i < db_info->field_count; i++) { /* for each columns. */
                 sqlite3_bind_text(stmt, index++, string_rawstr(k), string_len(k), NULL);
                 d("settext %d:%s\n", index - 1, string_rawstr(k));
             }
         }
     }
 
+    /* retrieve search results. */
     while (SQLITE_ROW == (r = sqlite3_step(stmt))) {
         Ticket* t = list_new_element(tickets);
-        /* 検索結果を受け取る */
         t->project_id = sqlite3_column_int(stmt, 0);
         t->id = sqlite3_column_int(stmt, 1);
         d("get %d:%d\n", t->project_id, t->id);
+
+        set_project_code_by_ticket(t, projects);
+        t->project_name = db_top_get_project_name(db, t, t->project_name);
+        t->title = db_top_get_title(db, t, t->title);
+
         list_add(tickets, t);
     }
     if (SQLITE_DONE != r)
         goto error;
+
+    /* detatch dbs. */
+
+    foreach (it, projects) {
+        char sql[DEFAULT_LENGTH];
+        ProjectInfo* pi = it->element;
+
+        if (count < start || count > start + offset) continue; /* ignore cause out of target range. */
+        if (pi->id == 1) continue; /* ignore cause top sub project. */
+        if (pi->deleted) continue; /* ignore cause deleted sub project. */
+        sprintf(sql, "detach db%d", pi->id);
+        d("%s\n", sql);
+        exec_query(db, sql, COLUMN_TYPE_END);
+    }
+
     sqlite3_finalize(stmt);
 
-    list_free(keywords_a);
-    list_free(sqls_a);
-    string_free(sql_search_a);
-
     list_free(db_infos_a);
-    /* チケットのタイトルとプロジェクト名を取得する */
-    foreach (it, tickets) {
-        Ticket* t = it->element;
-        set_project_code_by_ticket(t, projects_a);
-        t->project_name = db_top_get_project_name(db, t, t->project_name);
-        t->title = db_top_get_title(db, t, t->title);
-
-    }
-    list_free(projects_a);
+    string_free(sql_search_a);
     return tickets;
 
 ERROR_LABEL(db->handle)
 }
+List* db_top_search(Database* db, char* q, List* tickets)
+{
+    List* projects_a;
+    List* sqls_a;
+    List* keywords_a;
+    int i;
+
+    list_alloc(keywords_a, String, string_new, string_free);
+    keywords_a = parse_keywords(keywords_a, q);
+    /* search for sub projects. */
+    list_alloc(projects_a, ProjectInfo, project_info_new, project_info_free);
+    list_alloc(sqls_a, String, string_new, string_free);
+    projects_a = db_top_get_all_project_infos(db, projects_a);
+
+    for (i = 0; i < projects_a->size; i += 10) {
+        tickets = get_result_part(db, projects_a, sqls_a, keywords_a, i, 10, tickets);
+    d("aa 0\n");
+    }
+
+    list_free(keywords_a);
+    list_free(sqls_a);
+    list_free(projects_a);
+    return tickets;
+
+}
+
 String* db_top_get_top_project_name(String* project_name)
 {
     Database* top_db_a = db_init("top");
@@ -288,6 +325,7 @@ String* db_top_get_top_project_name(String* project_name)
 
 ERROR_LABEL(top_db_a->handle)
 }
+
 String* db_top_get_project_name(Database* db, Ticket* t, String* project_name)
 {
     String* sql_a = string_new();
@@ -311,6 +349,7 @@ String* db_top_get_project_name(Database* db, Ticket* t, String* project_name)
 
 ERROR_LABEL(db->handle)
 }
+
 String* db_top_get_title(Database* db, Ticket* t, String* title)
 {
     String* sql_a = string_new();
@@ -335,6 +374,7 @@ String* db_top_get_title(Database* db, Ticket* t, String* title)
 
 ERROR_LABEL(db->handle)
 }
+
 String* db_top_get_locale(String* locale)
 {
     Project* top_project_a = project_new();
@@ -348,6 +388,7 @@ String* db_top_get_locale(String* locale)
     project_free(top_project_a);
     return locale;
 }
+
 void db_top_set_locale()
 {
     String* locale_a = string_new();
